/*
  Copyright (C) 2001 Kimmo Pekkola

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
/*
  $Header: //rainbox/cvsroot/Rainmeter/ExamplePlugin/ExamplePlugin.c,v 1.2 2001/12/23 10:04:51 rainy Exp $

  $Log: ExamplePlugin.c,v $
  Revision 1.2  2001/12/23 10:04:51  rainy
  Added ID to the interface.

  Revision 1.1  2001/10/28 09:05:05  rainy
  Inital version

*/

#include <windows.h>
#include <math.h>

/* The exported functions */
__declspec( dllexport ) UINT Initialize(HMODULE instance, LPCTSTR iniFile, LPCTSTR section, UINT id);
__declspec( dllexport ) void Finalize(HMODULE instance, UINT id);
__declspec( dllexport ) UINT Update(UINT id);

/* Couple of globals */
static UINT g_Phase = 100;
static UINT g_CurrentPhase = 0;

/*
  This function is called when the measure is initialized.
  The function must return the maximum value that can be measured. 
  The return value can also be 0, which means that Rainmeter will
  track the maximum value automatically. The parameters for this
  function are:

  instance  The instance of this DLL
  iniFile   The name of the ini-file (usually Rainmeter.ini)
  section   The name of the section in the ini-file for this measure
  id        The identifier for the measure. This is used to identify the measures that use the same plugin.
*/
UINT Initialize(HMODULE instance, LPCTSTR iniFile, LPCTSTR section, UINT id)
{
	/* Read our own settings from the ini-file */
	g_CurrentPhase = GetPrivateProfileInt(section, "PhaseShift", 0, iniFile);
	g_Phase = GetPrivateProfileInt(section, "Phase", 100, iniFile);

	return 1000;	/* We'll return values from 0 to 1000 */
}

/*
  This function is called when new value should be measured.
  The function returns the new value.
*/
UINT Update(UINT id)
{
	/* 
	  This measure doesn't measure anything. It's just an
	  example how to create Rainmeter plugins. We'll just
	  return sine curve so that the meter shows something.
	*/

	double value = 6.283185 * ((double)g_CurrentPhase / (double)g_Phase);
	value = sin(value);
	
	g_CurrentPhase++;
	if(g_CurrentPhase > g_Phase)
	{
		g_CurrentPhase = 0;
	}

	return (UINT)((value + 1.0) * 500.0);
}

/*
  If the measure needs to free resources before quitting.
  The plugin can export Finalize function, which is called
  when Rainmeter quits (or refreshes).
*/
void Finalize(HMODULE instance, UINT id)
{
	/* Nothing to do here */
}
